@@ -295,7 +295,7 @@ const struct Item	*form_data;    /* form item of a form to submit */
 
 	       /* check for redirect */
 	       for(cur_header=0; cur_header<res->handle.http->headers.count; ++cur_header) {    /* all headers */
-		  if(strcmp(res->handle.http->headers.header[cur_header].name, "Location")==0) {    /* found */
+		  if(strcmp(res->handle.http->headers.header[cur_header].name, "location")==0) {    /* found */
 		     redir_header=&res->handle.http->headers.header[cur_header];
 		     break;    /* don't search further */
 		  }
