@@ -13,7 +13,7 @@
  * as argument), and either dump it and quit immediatly (with --dump), or
  * display it interactively and load new files (requested by user) in a loop
  */
-#include <curses.h>
+#include <ncursesw/curses.h>
 #include <errno.h>
 #include <locale.h>
 #include <stdio.h>
