@@ -7,10 +7,11 @@
  * pager.c -- Interactive viewing module.
  *
  * (C) 2001, 2002, 2003 antrik
+ *     2007 Evan Gates (emg)
  *
  * Displays output page and handles user commands.
  */
-#include <curses.h>
+#include <ncursesw/curses.h>
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -38,6 +39,7 @@ static int active_end(const struct Page *page, int pos);    /* find last active
 static void set_cursor(struct Page *page, int x, int y);    /* set new cursor position */
 
 static int find_link(const struct Page *page, int start_x, int start_y, int end_x, int end_y, int end_line, enum Match_type match_type);    /* find link matching criteria */
+static int label_links(struct Page *page);
 
 static void activate_link(struct Page *page, int change_link);    /* set active link */
 static void scroll_to(struct Page *page, int new_line);    /* adjust visible part of page */
@@ -194,6 +196,106 @@ enum Match_type		match_type;    /* take first match or last match (list directio
    return -1;    /* no valid link in range */
 }
 
+/* emg, antrik --> */
+
+/*
+ * Create link labels and let user select one.
+ *
+ * Creates labels (short strings of usually one or two letters, depending on
+ * how much are needed) for each link in the current active screen area,
+ * painting it right over the beggining of the link.
+ *
+ * Afterwards the user needs to enter the necessary number of letters, and the
+ * corresponding link gets selected. Aborts if an invalid character (anything
+ * but a lower-case letter) gets entered, or a non-existing label.
+ *
+ * Returns true if a link got selected, false otherwise.
+ */
+
+static int label_links(page)
+struct Page	*page;   /* page to search */
+{
+   const int	start_y=active_start(page, 0);
+   const int	end_line=active_end(page,0);
+
+   int	first, last;    /* first/last link on page */
+   int	taglen;
+
+   first=find_link(page, -1, start_y, -1, -1, end_line, FIND_FIRST);
+   last=find_link(page, -1, start_y, -1, -1, end_line, FIND_LAST);
+
+   if(first==-1)   /* no links on the page */
+       return 0;
+
+   /* how many letters per label do we need? */
+   {
+      int	num=last-first;    /* highest label number, 0-based (i.e. one less than number of links on active screen area) */
+      for(taglen=1; num/=26; ++taglen)    /* start with one letter, and continue adding more as long as another letter will still be significant (highest used value > 0) */
+	 ;
+   }
+
+   /* create labels */
+   {
+      char	tag[taglen+1];    /* 0-terminated */
+      int	current;
+
+      /* first label is all 'a' */
+      memset(tag, 'a', taglen);
+      tag[taglen]='\0';
+
+      set_color(color_map[TM_SYS] | 0x08);
+      for(current=first; current<=last; ++current) {
+	 const struct Link	*link=get_link(page->layout, current);
+
+	 int	i;
+
+	 mvprintw(link->y_start - page->pager_pos, link->x_start, "%s", tag);
+
+	 /* count up */
+	 ++tag[taglen-1];    /* increment */
+	 for(i=taglen-1; i>0; --i) {    /* all letters except highest */
+	    if(tag[i]>'z') {    /* overflow to next-higher letter */
+	       tag[i]='a';
+	       ++tag[i-1];
+	    }
+	 }
+      }
+   }    /* create labels */
+
+   /* select label */
+   {
+      int	letter;
+      int	selected;
+
+      for(letter=0, selected=0; letter<taglen; ++letter) {
+	 int	in;
+
+	 enable_winch();   /* allow SIGWINCH during keywait */
+	 in=mvgetch(page->cursor_y - page->pager_pos, page->cursor_x);
+	 hold_winch();
+
+	 if(in<'a' || in>'z') {    /* not a valid label character */
+	    selected=-1;    /* -> don't activate anything */
+	    if(in==KEY_RESIZE)
+	       ungetch(in);    /* handle SIGWINCH after returning to pager */
+	    break;
+	 }
+
+	 selected=selected*26+in-'a';
+      }
+
+      render(page->layout, 0, page->pager_pos, 0, 0, COLS, LINES, 0);    /* repaint whole screen (clear labels) */
+
+      if(selected>=0 && selected<=last-first) {    /* valid label selected */
+	 activate_link(page, selected+first);
+	 return 1;
+      } else
+	 return 0;
+   }    /* select label */
+}
+
+/* <-- emg, antrik */
+
 /*
  * set active link
  *
@@ -720,6 +822,16 @@ struct Page	*page;
 		  flash();
 	       break;
 
+	    /* Evan --> */
+
+	    /* label links */
+	    case 'l':
+	       if(label_links(page))
+		  quit=RET_LINK;
+	       break;
+
+	    /* <-- Evan */
+
 	    /* show link URL */
 	    case 'u':
 	       if(page->active_link>=0)    /* some link active */
