@@ -12,7 +12,7 @@
  * all content. (Text inside the elements.)
  */
 #include <ctype.h>
-#include <curses.h>
+#include <ncursesw/curses.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
