@@ -587,3 +587,30 @@ const char		*form;
 
    return url;
 }
+
+/* Returns a string with %xx URL escape codes from the source string replaced by the respective characters. The result is newly allocated; the source remains untouched. */
+char *url_unescape(escaped)
+const char *escaped;
+{
+   char *rpos, *wpos, chr;
+   char *result;
+
+   result=malloc(strlen(escaped)+1);
+   if(result==NULL) {
+      fprintf(stderr, "memory allocation error while unescaping path (in function url_unescape)\n");
+      exit(1);
+   }
+
+   wpos=result; rpos=(char *)escaped;
+   do {
+      if(*rpos=='%') {
+	 int code;
+	 sscanf(rpos+1, "%2x", &code);
+	 chr=(unsigned char)code;
+	 rpos+=3;
+      } else
+	 chr=*rpos++;
+   } while((*wpos++=chr));
+
+   return result;
+}
